package Plugins::DRS::Plugin;

use strict;

use vars qw($VERSION);
use Digest::SHA1;
use HTTP::Request;

use Slim::Networking::SimpleAsyncHTTP;
use Slim::Networking::SqueezeNetwork;
use Slim::Utils::Log;
use Slim::Utils::Prefs;

use constant SEARCH_URL => 'http://www.herger.net/cgi-bin/getartwork.cgi';

my $log = Slim::Utils::Log->addLogCategory({
	'category'     => 'plugin.drs',
	'description'  => 'PLUGIN_DRS_META',
});

my $prefs = preferences('server');

use constant cacheTTL => 15;

my $icons = {
	drs1 => 'http://upload.wikimedia.org/wikipedia/commons/thumb/d/d3/Radio_SRF_1.svg/500px-Radio_SRF_1.svg.png',
	drs2 => 'http://upload.wikimedia.org/wikipedia/commons/thumb/a/a2/Radio_SRF_2_Kultur.svg/500px-Radio_SRF_2_Kultur.svg.png',
	drs3 => 'http://upload.wikimedia.org/wikipedia/commons/thumb/e/ee/Radio_SRF_3.svg/500px-Radio_SRF_3.svg.png',
	drs4 => 'http://upload.wikimedia.org/wikipedia/commons/thumb/e/e3/Radio_SRF_4_News.svg/500px-Radio_SRF_4_News.svg.png',
	drsmw=> 'http://upload.wikimedia.org/wikipedia/commons/thumb/7/79/Radio_SRF_Musikwelle.svg/500px-Radio_SRF_Musikwelle.svg.png',
	drsvirus => 'http://upload.wikimedia.org/wikipedia/commons/thumb/a/ad/Radio_SRF_Virus.svg/500px-Radio_SRF_Virus.svg.png',
	rsp  => 'http://a2.mzstatic.com/us/r1000/074/Purple/f4/0f/af/mzl.qaiwrwpe.png',
	rsj  => 'http://upload.wikimedia.org/wikipedia/de/thumb/c/c0/Radio_Swiss_Jazz_Logo.svg/500px-Radio_Swiss_Jazz_Logo.svg.png',
	'1ere' => 'http://upload.wikimedia.org/wikipedia/fr/d/df/La_Premi%C3%A8re.png',
	'espace' => 'http://upload.wikimedia.org/wikipedia/de/thumb/4/48/Logo_Espace2.svg/500px-Logo_Espace2.svg.png',
	couleur => 'http://www.psiaudio.com/sites/default/files/images/IMG_3059Couleur3.JPG',
	reteuno => 'http://upload.wikimedia.org/wikipedia/it/thumb/c/cc/RSI_Rete_Uno_2013.svg/500px-RSI_Rete_Uno_2013.svg.png',
	retedue => 'http://upload.wikimedia.org/wikipedia/it/thumb/f/f4/RSI_Rete_Due_2013.svg/500px-RSI_Rete_Due_2013.svg.png',
	retetre => 'http://upload.wikimedia.org/wikipedia/it/8/88/639px-RSI_Rete_Tre_2009.svg-1-.png',
};

my $canArtwork = {
	drs1 => 1,
	drs2 => 1,
	drs3 => 1,
	drs4 => 0,
	drsmw=> 1,
	drsvirus => 1,
	rsp  => 1,
	rsj  => 1,
	'1ere' => 1,
	'espace' => 1,
	'couleur' => 1,
	reteuno => 1,
	retedue => 1,
	retetre => 1,
};

my $urlParseRegex = qr/(?:srg-ssr|streamserver|skypro|swisstxt).*(drs(?:[1-4]|virus|mw)|rete(?:uno|due|tre)|regi_[a-z_]+|drs_event|\b(?:rsp|1ere|espace|rsj)\b|\bcouleur)/i;

my $imageProxy;

sub getDisplayName {
	return 'PLUGIN_DRS_META';
}

sub initPlugin {
	my $class = shift;
	
	$VERSION = $class->_pluginDataFor('version');

	Slim::Formats::RemoteMetadata->registerParser(
		match => $urlParseRegex,
		func  => \&parser,
	);

	# support for new LMS 7.8 local image proxy
	# register custom image proxy parser to optimize artwork download
	eval {
		require Slim::Web::ImageProxy;
		
		$imageProxy = sub {
			my ($url, $spec) = @_;
			
			main::DEBUGLOG && $log->debug("Radio SRF artwork - let's get the smallest version fitting our needs: $url, $spec");
			
			my ($url2, $sizes) = $url =~ /(.*)\?sizes=(.*)$/;
			
			return $url unless $url2 && $sizes;
			
			my %sizeMap = map {
				/(\d+).*/ && $1 => $_
			} split(/\|/, $sizes);
			
			my $size = Slim::Web::ImageProxy->getRightSize($spec, \%sizeMap) || '_';
			$url2 =~ s/\/_\//\/$size\//;
			
			main::DEBUGLOG && $log->debug("Going to get $url2");
			
			return $url2;
		};

		Slim::Web::ImageProxy->registerHandler(
			match => qr/\/_\/.*\?sizes=[\d\|sx]+$/,
			func  => $imageProxy,
		);
		
		main::DEBUGLOG && $log->debug("Successfully registered image proxy for SRF artwork");
	}
}

sub _pluginDataFor {
	my $class = shift;
	my $key   = shift;

	my $pluginData = Slim::Utils::PluginManager->dataForPlugin($class);

	if ($pluginData && ref($pluginData) && $pluginData->{$key}) {
		return $pluginData->{$key};
	}

	return undef;
}


sub parser {
	my ( $client, $url, $metadata ) = @_;
	
	if ($metadata =~ (/StreamTitle=\'(.*?)\'(?:;|$)/)) {
		my $titleInfo = $1;
		
		# what station?
		$url =~ $urlParseRegex;
		my $station = lc($1 || 'drs3');
		
		main::DEBUGLOG && $log->is_debug && $log->debug("Icy metadata received for $station: $titleInfo");

		my ($artist, $title) = split(' - ', $titleInfo);
		
		my $info = {
			icon   => $icons->{$station},
			cover  => $icons->{$station},
			artist => undef,
			title  => $titleInfo,
		};
		
		if ($title && $artist) {
			
			$info->{artist} = _cleanup($artist);
			$info->{title} = _cleanup($title);

			getArtwork( $client, $station, $info );
		}

		my $song = $client->controller()->songStreamController()->song();
		
		if ($song) {
			$song->pluginData( wmaMeta => $info );
	
			Slim::Control::Request::notifyFromArray( $client, [ 'newmetadata' ] );
			Slim::Music::Info::setDelayedTitle( $client, $url, $titleInfo );
		}

		return 1;
	}
	
	main::DEBUGLOG && $log->is_debug && $log->debug("No metadata set for $url");
	
	return 0;
}

sub getArtwork {
	my ( $client, $station, $info ) = @_;
	
	return unless $canArtwork->{$station};
	return if !$info->{artist} || $info->{artist} =~ /(?:\Q$station\E|la 1.re|radio swiss)/i;

	main::DEBUGLOG && $log->is_debug && $log->debug("Let's get some artwork...");
	
	$info->{title} =~ s/(?:20|19)\d\d$//;	# RETE is adding the year or something to every title
	
	my $args = '?track=' . URI::Escape::uri_escape_utf8( $info->{title} || '' )
		. '&artist=' . URI::Escape::uri_escape_utf8( $info->{artist} || '' ); 

	my $http = Slim::Networking::SimpleAsyncHTTP->new(
		sub {
			my $html = shift;

			if ( $html && $html->content && $html->content =~ /^http/ ) {
				my $imageUrl = $html->content;
				$imageUrl =~ s/\?sizes=.*$// unless $imageProxy;
				
				main::DEBUGLOG && $log->is_debug && $log->debug("Found title artwork: " . $imageUrl);
				
				#$meta->{$station}->{icon} = $meta->{$station}->{cover} = $imageUrl;

				$client->playingSong->pluginData( wmaMeta => {
					icon   => $imageUrl,
					cover  => $imageUrl,
					artist => $info->{artist},
					title  => $info->{title},
				} );

				Slim::Control::Request::notifyFromArray( $client, [ 'newmetadata' ] );
			}
			elsif ( $html && $html->content ) {
				
				main::DEBUGLOG && $log->is_debug && $log->debug("Nothing found: " . $html->content);
				
			}
			else {
				
				main::DEBUGLOG && $log->is_debug && $log->debug(Data::Dump::dump($html));
				
			}
		},
		sub {
			logError(Data::Dump::dump(shift));
		},
	);
	
	$http->get( SEARCH_URL . $args );
	
	return 1;
}

sub _cleanup {
	my $s = shift;
	
	$s =~ s/ \(CH\)\s*$//;
	
	utf8::decode($s);
	
	# only mess with upper/lowercase if we don't have any lower case yet
	return $s if $s =~ /[a-z]+/;
	
	# uppercase all first characters
	$s =~ s/([\w'\`[:alpha:]äöüéàèçôî]+)/\u\L$1/ig;
		
	# fallback in case locale is wrong 
	$s =~ tr/äöüàéèçôî/ÄÖÜÀÉÈÇÔÎ/;

	$s =~ s/^\s+//;
	$s =~ s/[\:\s]+$//;
	
	return $s;
}

1;